#include <dbsim.h>
#include <node.h>
#include "bufmgr.h"
#include "bfbucket.h"


// Manages free store of hash-table buckets for buffer manager.
// Overloads "new" so that new first checks to see if there are
// any allocated but currently unused buckets around; if there
// are, just returns one, if not, malloc()'s a new one.
// Similarly, "free" just adds the freed bucket to the free list.

BFBucket* BFBucket::freeBucket = NULL;       // no free buckets initially

BFBucket::BFBucket() {
  bufIndex = ILLEGAL_BUFINDEX;   
  next = 0;
  pid.diskId = ILLEGAL_DISK;
  pid.pageId = ILLEGAL_PAGE;
  ioComplete = NIL;
  inTransit = FALSE;
}


void* BFBucket::operator new(size_t size)
{
    BFBucket *newBucket;

    if (freeBucket == NULL) {
        // allocate a new bucket
        newBucket = (BFBucket *) new char[size];
      } else {
        // reuse an existing bucket
        newBucket = freeBucket;
        freeBucket = freeBucket->next;
      }
    return newBucket;
  };


void BFBucket::operator delete(void* oldBucket, size_t)
{
    // put the bucket on the free list
    ((BFBucket *) oldBucket)->next = freeBucket;
    freeBucket = (BFBucket *) oldBucket;
  };



