
#include <dbsim.h>
#include <object.h>
#include <node.h>
#include <language.h>
#include <db.h>

//---------------------------------------------------------------------------
// class implementation for FileScans
//---------------------------------------------------------------------------

FileScan::FileScan(File* filePtr, Node* nodePtr, int diskNo)
{
    // prepare to scan all fragments of file on node
    file = filePtr;
    node = nodePtr;
    if (diskNo == ALL_FRAGMENTS) {
	allFrags = TRUE;
        curDisk  = 0;
    } else {
	allFrags = FALSE;
        curDisk  = diskNo;
    };
    curFrag = file->fragment[node->id][file->storageDisk(curDisk)];
    curPage = -1;
}


int FileScan::nextPage(int& physPage, int& disk, int& numTuples)
{
    // return immediately if scan is past last page of last fragment
    if ( (curPage == curFrag->numPages-1) &&
         ((curDisk == file->numDisks-1) || (!allFrags)) ) {
        disk = file->storageDisk(curDisk); // set return value
	return(END_OF_SCAN);
    }

    // advance scan state to next page on this node
    curPage++;
    if (curPage == curFrag->numPages) {
	// time to switch to next fragment
	curPage = 0;
	curDisk++;
	if (curDisk == file->numDisks) {
	    return (BAD_PAGE);
	}
        curFrag = file->fragment[node->id][file->storageDisk(curDisk)];
    }

    // setup return variable values and return them
    physPage = curFrag->lookupPage(curPage);
    disk = file->storageDisk(curDisk);
    if (curPage == curFrag->numPages-1) {
	numTuples = curFrag->lastPgTuples;
    } else {
	numTuples = curFrag->tuplesPerPage;
    }
    return (OK);
}

