
#include <dbsim.h>
#include <object.h>
#include <node.h>
#include <language.h>
#include <db.h>
#include <strings.h>
#include <math.h>

//---------------------------------------------------------------------------
// class implementation for IndexFrag
//---------------------------------------------------------------------------

IndexFrag::IndexFrag(FileFrag* myData, int keySize, int myIndexType)
{
    // variables for computing index characteristics
    int numPages, nextPage, extraEntries;

    // check for possibility of an empty fragment being indexed
    if (myData->numTuples == 0) {
	printf("ERROR in IndexFrag:  Indexing an empty fragment!\n");
	return;
    }

    // compute index characteristics, with fanout based on each index
    // page storing a count, N+1 pointers, and N search keys
    data = myData;
    indexType = myIndexType;
    fanout = ((PageSize - PTR_SIZE - sizeof(int)) / (keySize + PTR_SIZE)) + 1;
    numEntries = data->numTuples;
    numLevels = nextPage = 0;
    numPages = numEntries;
    do {
	extraEntries = numPages % fanout;
	numPages = (numPages / fanout) + (extraEntries != 0);
        firstPage[numLevels] = nextPage;
        lastFanout[numLevels] = (extraEntries == 0) ? fanout : extraEntries;
	nextPage += numPages;
	numLevels++;
    } while ((numPages > 1) && (numLevels < MAX_HEIGHT));

    // check for possibility of an overly tall index
    if (numPages > 1) {
	printf("ERROR in IndexFrag:  Index too deep!\n");
	return;
    }

    // now allocate a file fragment big enough for the index
    nodes = new FileFrag(data->fragNo, data->disk, fanout, fanout*nextPage);
}



IndexFrag::~IndexFrag()
{
    delete nodes;
}



void IndexFrag::print()
{
    printf("\t    IndexFrag %d:\n", data->fragNo);
    printf("\t\tnumEntries = %d, fanout = %d, numLevels = %d\n",
            numEntries, fanout, numLevels);
    printf("\t\tfirstPage = { ");
    for (int fp = 0; fp < numLevels; fp++) {
	printf("%d ", firstPage[fp]);
    }
    printf("}\n");
    printf("\t\tlastFanout = { ");
    for (int lf = 0; lf < numLevels; lf++) {
	printf("%d ", lastFanout[lf]);
    }
    printf("}\n");
}

